@@ -9,6 +9,7 @@ import matplotlib
 import matplotlib.ft2font
 import matplotlib.pyplot as plt
 import pytest
+from helpers import skip_if_format_unsupported
 from packaging.version import Version
 
 MPL_VERSION = Version(matplotlib.__version__)
@@ -119,11 +120,10 @@ def test_fail():
 """
 
 
-def test_fails(tmpdir):
-
-    test_file = tmpdir.join('test.py').strpath
-    with open(test_file, 'w') as f:
-        f.write(TEST_FAILING)
+def test_fails(tmp_path):
+    test_file = tmp_path / "test.py"
+    test_file.write_text(TEST_FAILING)
+    test_file = str(test_file)
 
     # If we use --mpl, it should detect that the figure is wrong
     code = call_pytest(['--mpl', test_file])
@@ -146,16 +146,15 @@ def test_output_dir():
 """
 
 
-def test_output_dir(tmpdir):
-    test_file = tmpdir.join('test.py').strpath
-    with open(test_file, 'w') as f:
-        f.write(TEST_OUTPUT_DIR)
+def test_output_dir(tmp_path):
+    test_file = tmp_path / "test.py"
+    test_file.write_text(TEST_OUTPUT_DIR)
 
-    output_dir = tmpdir.join('test_output_dir')
+    output_dir = tmp_path / "test_output_dir"
 
     # When we run the test, we should get output images where we specify
     code = call_pytest([f'--mpl-results-path={output_dir}',
-                        '--mpl', test_file])
+                        '--mpl', str(test_file)])
 
     assert code != 0
     assert output_dir.exists()
@@ -174,13 +173,14 @@ def test_gen():
 """
 
 
-def test_generate(tmpdir):
+def test_generate(tmp_path):
 
-    test_file = tmpdir.join('test.py').strpath
-    with open(test_file, 'w') as f:
-        f.write(TEST_GENERATE)
+    test_file = tmp_path / "test.py"
+    test_file.write_text(TEST_GENERATE)
+    test_file = str(test_file)
 
-    gen_dir = tmpdir.mkdir('spam').mkdir('egg').strpath
+    gen_dir = tmp_path / "spam" / "egg"
+    gen_dir.mkdir(parents=True)
 
     # If we don't generate, the test will fail
     assert_pytest_fails_with(['--mpl', test_file], 'Image file not found for comparison test')
@@ -280,7 +280,7 @@ class TestClassWithTestCase(TestCase):
 # hashlib
 
 @pytest.mark.skipif(not hash_library.exists(), reason="No hash library for this mpl version")
-@pytest.mark.mpl_image_compare(hash_library=hash_library)
+@pytest.mark.mpl_image_compare(hash_library=hash_library, deterministic=True)
 def test_hash_succeeds():
     fig = plt.figure()
     ax = fig.add_subplot(1, 1, 1)
@@ -291,7 +291,7 @@ def test_hash_succeeds():
 TEST_FAILING_HASH = rf"""
 import pytest
 import matplotlib.pyplot as plt
-@pytest.mark.mpl_image_compare(hash_library=r"{fail_hash_library}")
+@pytest.mark.mpl_image_compare(hash_library=r"{fail_hash_library}", deterministic=True)
 def test_hash_fails():
     fig = plt.figure()
     ax = fig.add_subplot(1,1,1)
@@ -300,11 +300,10 @@ def test_hash_fails():
 """
 
 
-def test_hash_fails(tmpdir):
-
-    test_file = tmpdir.join('test.py').strpath
-    with open(test_file, 'w', encoding='ascii') as f:
-        f.write(TEST_FAILING_HASH)
+def test_hash_fails(tmp_path):
+    test_file = tmp_path / "test.py"
+    test_file.write_text(TEST_FAILING_HASH, encoding="ascii")
+    test_file = str(test_file)
 
     # If we use --mpl, it should detect that the figure is wrong
     output = assert_pytest_fails_with(['--mpl', test_file], "doesn't match hash FAIL in library")
@@ -329,7 +328,7 @@ TEST_FAILING_HYBRID = rf"""
 import pytest
 import matplotlib.pyplot as plt
 @pytest.mark.mpl_image_compare(hash_library=r"{fail_hash_library}",
-                               tolerance=2)
+                               tolerance=2, deterministic=True)
 def test_hash_fail_hybrid():
     fig = plt.figure()
     ax = fig.add_subplot(1,1,1)
@@ -339,11 +338,11 @@ def test_hash_fail_hybrid():
 
 
 @pytest.mark.skipif(ftv != '261', reason="Incorrect freetype version for hash check")
-def test_hash_fail_hybrid(tmpdir):
+def test_hash_fail_hybrid(tmp_path):
 
-    test_file = tmpdir.join('test.py').strpath
-    with open(test_file, 'w', encoding='ascii') as f:
-        f.write(TEST_FAILING_HYBRID)
+    test_file = tmp_path / "test.py"
+    test_file.write_text(TEST_FAILING_HYBRID, encoding="ascii")
+    test_file = str(test_file)
 
     # Assert that image comparison runs and fails
     output = assert_pytest_fails_with(['--mpl', test_file,
@@ -381,18 +380,18 @@ def test_hash_fails():
 
 
 @pytest.mark.skipif(ftv != '261', reason="Incorrect freetype version for hash check")
-def test_hash_fail_new_hashes(tmpdir):
+def test_hash_fail_new_hashes(tmp_path):
     # Check that the hash comparison fails even if a new hash file is requested
-    test_file = tmpdir.join('test.py').strpath
-    with open(test_file, 'w', encoding='ascii') as f:
-        f.write(TEST_FAILING_NEW_HASH)
+    test_file = tmp_path / "test.py"
+    test_file.write_text(TEST_FAILING_NEW_HASH, encoding="ascii")
+    test_file = str(test_file)
 
     # Assert that image comparison runs and fails
     assert_pytest_fails_with(['--mpl', test_file,
                               f'--mpl-hash-library={fail_hash_library}'],
                              "doesn't match hash FAIL in library")
 
-    hash_file = tmpdir.join('new_hashes.json').strpath
+    hash_file = tmp_path / "new_hashes.json"
     # Assert that image comparison runs and fails
     assert_pytest_fails_with(['--mpl', test_file,
                               f'--mpl-hash-library={fail_hash_library}',
@@ -412,11 +411,10 @@ def test_hash_missing():
 """
 
 
-def test_hash_missing(tmpdir):
-
-    test_file = tmpdir.join('test.py').strpath
-    with open(test_file, 'w') as f:
-        f.write(TEST_MISSING_HASH)
+def test_hash_missing(tmp_path):
+    test_file = tmp_path / "test.py"
+    test_file.write_text(TEST_MISSING_HASH)
+    test_file = str(test_file)
 
     # Assert fails if hash library missing
     assert_pytest_fails_with(['--mpl', test_file, '--mpl-hash-library=/not/a/path'],
@@ -439,41 +437,36 @@ def plot():
     ax = fig.add_subplot(1,1,1)
     ax.plot([1,2,2])
     return fig
-@pytest.mark.mpl_image_compare
+@pytest.mark.mpl_image_compare(deterministic=True)
 def test_modified(): return plot()
-@pytest.mark.mpl_image_compare
+@pytest.mark.mpl_image_compare(deterministic=True)
 def test_new(): return plot()
-@pytest.mark.mpl_image_compare
+@pytest.mark.mpl_image_compare(deterministic=True)
 def test_unmodified(): return plot()
 """
 
 
 @pytest.mark.skipif(not hash_library.exists(), reason="No hash library for this mpl version")
-def test_results_always(tmpdir):
+def test_results_always(tmp_path):
+    test_file = tmp_path / "test.py"
+    test_file.write_text(TEST_RESULTS_ALWAYS)
+    results_path = tmp_path / "results"
+    results_path.mkdir()
 
-    test_file = tmpdir.join('test.py').strpath
-    with open(test_file, 'w') as f:
-        f.write(TEST_RESULTS_ALWAYS)
-    results_path = tmpdir.mkdir('results')
-
-    code = call_pytest(['--mpl', test_file, '--mpl-results-always',
+    code = call_pytest(['--mpl', str(test_file), '--mpl-results-always',
                         rf'--mpl-hash-library={hash_library}',
                         rf'--mpl-baseline-path={baseline_dir_abs}',
                         '--mpl-generate-summary=html,json,basic-html',
-                        rf'--mpl-results-path={results_path.strpath}'])
+                        rf'--mpl-results-path={results_path}'])
     assert code == 0  # hashes correct, so all should pass
 
     # assert files for interactive HTML exist
-    assert results_path.join('fig_comparison.html').exists()
-    assert results_path.join('styles.css').exists()
-    assert results_path.join('extra.js').exists()
+    assert (results_path / "fig_comparison.html").exists()
+    assert (results_path / "styles.css").exists()
+    assert (results_path / "extra.js").exists()
 
-    comparison_file = results_path.join('fig_comparison_basic.html')
-    with open(comparison_file, 'r') as f:
-        html = f.read()
-
-    json_file = results_path.join('results.json')
-    with open(json_file, 'r') as f:
+    html = (results_path / "fig_comparison_basic.html").read_text()
+    with (results_path / "results.json").open("r") as f:
         json_results = json.load(f)
 
     # each test, and which images should exist
@@ -494,7 +487,7 @@ def test_results_always(tmpdir):
 
         for image_type in ['baseline', 'result-failed-diff', 'result']:
             image = f'{test_name}/{image_type}.png'
-            image_exists = results_path.join(*image.split('/')).exists()
+            image_exists = (results_path / image).exists()
             json_image_key = f"{image_type.split('-')[-1]}_image"
             if image_type in exists:  # assert image so pytest prints it on error
                 assert image and image_exists
@@ -553,11 +546,11 @@ class TestClassWithTestCase(TestCase):
     TEST_FAILING_CLASS_SETUP_METHOD,
     TEST_FAILING_UNITTEST_TESTCASE,
 ])
-def test_class_fail(code, tmpdir):
+def test_class_fail(code, tmp_path):
 
-    test_file = tmpdir.join('test.py').strpath
-    with open(test_file, 'w') as f:
-        f.write(code)
+    test_file = tmp_path / "test.py"
+    test_file.write_text(code)
+    test_file = str(test_file)
 
     # Assert fails if hash library missing
     assert_pytest_fails_with(['--mpl', test_file, '--mpl-hash-library=/not/a/path'],
@@ -670,3 +663,39 @@ def test_user_function_raises(pytester, runpytest_args):
     result = pytester.runpytest(*runpytest_args)
     result.assert_outcomes(failed=1)
     result.stdout.fnmatch_lines("FAILED*ValueError*User code*")
+
+
+@pytest.mark.parametrize('use_hash_library', (False, True))
+@pytest.mark.parametrize('passes', (False, True))
+@pytest.mark.parametrize("file_format", ['eps', 'pdf', 'png', 'svg'])
+def test_formats(pytester, use_hash_library, passes, file_format):
+    """
+    Note that we don't test all possible formats as some do not compress well
+    and would bloat the baseline directory.
+    """
+    skip_if_format_unsupported(file_format, using_hashes=use_hash_library)
+    if use_hash_library and not hash_library.exists():
+        pytest.skip("No hash library for this mpl version")
+
+    pytester.makepyfile(
+        f"""
+        import os
+        import pytest
+        import matplotlib.pyplot as plt
+        @pytest.mark.mpl_image_compare(baseline_dir=r"{baseline_dir_abs}",
+                                       {f'hash_library=r"{hash_library}",' if use_hash_library else ''}
+                                       tolerance={DEFAULT_TOLERANCE},
+                                       deterministic=True,
+                                       savefig_kwargs={{'format': '{file_format}'}})
+        def test_format_{file_format}():
+            fig = plt.figure()
+            ax = fig.add_subplot(1, 1, 1)
+            ax.plot([{1 if passes else 3}, 2, 3])
+            return fig
+        """
+    )
+    result = pytester.runpytest('--mpl', '-rs')
+    if passes:
+        result.assert_outcomes(passed=1)
+    else:
+        result.assert_outcomes(failed=1)
